#!/usr/bin/python
# -*- coding: utf-8 -*-

'''跨平台公共库
'''

import xml

from testbase.conf import settings

from Element import MtControl
from Element import MtList

if settings.PLATFORM == 'Android':
    from qt4a.andrcontrols import *
    from qt4a.qpath import QPath
elif settings.PLATFORM == 'iOS':
    from qt4i.icontrols import *
    from Element import Element
    from qt4i.qpath import QPath
elif settings.PLATFORM == 'h5':
    from qt4a.andrcontrols import *
    from qt4a.qpath import QPath
    from qt4w.webcontrols import *
elif settings.PLATFORM == 'Cocos Creater':
    from qt4cc.cccontrols import CCScene, CCNode


else:
    raise NotImplementedError('Not supported platform %s' % settings.PLATFORM)


def inflate(self, name):
    locator = {}
    dirname = settings.PROJECT_ROOT
    path = os.path.join(os.path.join(os.path.abspath(dirname), "panel"), name)
    dom = xml.dom.minidom.parse(path)
    tree = dom.documentElement
    views = tree.getElementsByTagName("View")

    for view in views:
        dict = {}

        dict["root"] = self

        if view.hasAttribute("name"):
            locator[view.getAttribute("name").encode('utf-8')] = dict

        if settings.PLATFORM == 'Android':
            android = view.getElementsByTagName("android")[0]
            if android.hasAttribute("type"):
                dict["type"] = globals()[android.getAttribute("type")]
            if android.hasAttribute("locator"):
                dict["locator"] = QPath(android.getAttribute("locator").encode('utf-8'))
        elif settings.PLATFORM == 'iOS':
            ios = view.getElementsByTagName("ios")[0]
            if ios.hasAttribute("type"):
                dict["type"] = globals()[ios.getAttribute("type")]
            if ios.hasAttribute("locator"):
                dict["locator"] = QPath(ios.getAttribute("locator").encode('utf-8'))
            if ios.hasAttribute("root"):
                dict["root"] = ios.getAttribute("root")
        elif settings.PLATFORM == 'h5':
            h5 = view.getElementsByTagName("h5")[0]
            if h5.hasAttribute("type"):
                dict["type"] = globals()[h5.getAttribute("type")]
            if h5.hasAttribute("locator"):
                dict["locator"] = XPath(h5.getAttribute("locator").encode('utf-8'))
        elif settings.PLATFORM == 'Cocos Creator':
            cocos = view.getElementsByTagName("cocos")[0]
            if cocos.hasAttribute("type"):
                dict["type"] = globals()[cocos.getAttribute("type")]
            if cocos.hasAttribute("locator"):
                dict["locator"] = QPath(cocos.getAttribute("locator").encode('utf-8'))
        else:
            raise NotImplementedError('Not supported platform %s' % settings.PLATFORM)

    metisViews = tree.getElementsByTagName("MetisView")

    for metisView in metisViews:
        dict = {}

        dict["root"] = self
        dict["type"] = MtControl

        if metisView.hasAttribute("name"):
            locator[metisView.getAttribute("name").encode('utf-8')] = dict

    metisLists = tree.getElementsByTagName("MetisList")

    for metisList in metisLists:

        dict = {}
        dict["root"] = self
        dict["type"] = MtList

        if metisList.hasAttribute("name"):
            locator[metisList.getAttribute("name").encode('utf-8')] = dict

    return locator


if settings.PLATFORM == 'Android':
    pass

elif settings.PLATFORM == 'iOS':
    # TODO: iOS端完善
    pass
elif settings.PLATFORM == 'h5':
    pass
else:
    raise NotImplementedError('Not supported platform %s' % settings.PLATFORM)

if __name__ == "__main__":
    print inflate("test", "LoginPanel.xml")



# -*- coding: UTF-8 -*-

import time

from qt4a.andrcontrols import *
from qt4a.qpath import QPath

from OEDWindow import OEDWindow


class QQLoginPanel(OEDWindow):
    Activity = 'com.tencent.mobileqq.activity.LoginActivity'

    def __init__(self, app, leave_time=2):
        super(QQLoginPanel, self).__init__(app, leave_time=leave_time)
        self.update_locator({
            '登录': {'type': Button, 'root': self, 'locator': QPath('/Id="btn_login"')},
            '帐号': {'type': EditText, 'root': self, 'locator': QPath('/Type="bcck" && Id="0x20e"')},
            '密码': {'type': EditText, 'root': self, 'locator': QPath('/Id="password"')},
            '提交': {'type': TextView, 'root': self, 'locator': QPath('/Id="login"')},
            '同意': {'type': TextView, 'root': self, 'locator': QPath('/Text="同意"')},
        })

    def login(self, acc, pwd):
        print("帐号 %s" % acc)
        print("密码 %s" % pwd)
        acc = "384847349"
        pwd = "1qa2ws3ed"
        if self.wait_for_exist(timeout=5, interval=0.5):
            if self.Controls["同意"].exist():
                self.Controls["同意"].click()
                time.sleep(2)
            if (self.Controls["登录"].exist()):
                self.Controls["登录"].click()
            time.sleep(3)
            self.Controls["帐号"].send_text(acc)
            self.Controls["密码"].send_text(pwd)
            time.sleep(1)

            if (self.Controls["提交"].exist()):
                self.Controls["提交"].click()



# -*- coding: UTF-8 -*-
'''
OED Window类
'''
from qt4a.andrcontrols import *


class OEDWindow(Window):
    def __init__(self, app, **kwds):
        super(OEDWindow, self).__init__(app, wait_activity=True, **kwds)



# -*- coding:utf-8 -*-
'''Ke App类
'''

import time

from qt4i.app import App
from qt4i.device import Keyboard
from testbase.conf import settings


class OEDApp(App):
    '''KeApp for ios 负责被测应用的启动和初始化
    '''

    def __init__(self, device, app_name=settings.APP_BUNDLE_ID, trace_template=None, trace_output=None,
                 clear_data=False, choose_user_guide=False, **params):
        """
        APP应用（启动APP）
        :param device         : Device的实例对象
        :type device          : Device
        :param app_name       : APP的BundleID（例如：com.tencent.sng.test.gn）
        :type app_name        : str
        :param trace_template : trace模板（专项测试使用，功能测试默认为None即可）
        :type trace_template  : str
        :param trace_output   : teace存储路径（专项测试使用，功能测试默认为None即可）
        :type trace_output    : str
        :param clear_data   : 是否清理App数据（暂时没实现）
        :type clear_data    : boolean
        """
        App.__init__(self, device, app_name, trace_template, trace_output, **params)
        self.choose_user_guide = choose_user_guide
        self.re_install = True
        self.set_environment()
        self.start()
        self.keyboard = Keyboard(device)

        time.sleep(5)

    def set_environment(self):
        '''初始化自动处理Alert弹框应对规则

        :param: none
        :returns: none
        '''
        # 此规则用于处理预期内容但难以预期弹出时机的Alert框（注意国际化多国语言的情况）。
        # 配置策略后，只要Alert框命中策略，即按策略处理。例如指定点击取消或确定按钮。
        self.rules_of_alert_auto_handle = [

            # 推送通知
            {
                'message_text': '推送通知|Notifications',  # 支持正则表达式
                'button_text': '^好$|^Allow$|^允许$'  # 支持正则表达式
            },

            # ios10推送通知
            {
                'message_text': '发送通知|Notifications',  # 支持正则表达式
                'button_text': '^好$|^Allow$|^允许$'  # 支持正则表达式
            },

            # 测试账号Alert框（测试账号会弹出，曾经出现连弹多次）
            {
                'message_text': '测试号码',  # 支持正则表达式
                'button_text': '^确定$'  # 支持正则表达式
            },

            # 授权APP访问位置
            {
                'message_text': '位置|Location',
                'button_text': '^好$|^Allow$|^允许$'
            },

            # 授权APP访问相册
            {
                'message_text': '照片|Photos',
                'button_text': '^允许$|^好$|^OK$'
            },

            # 授权APP访问麦克风
            {
                'message_text': '麦克风',
                'button_text': '^允许$|^好$|^OK$'
            },

            # 授权APP访问相机
            {
                'message_text': '相机',
                'button_text': '^允许$|^好$|^OK$'
            },

            # 用于处理退出登录 (屏蔽退出登录的自动处理)
            {
                'message_text': '退出当前帐号',
            },

        ]

        # 此开关打开，用于处理不可预期内容且不可预期时机的Alert框
        # 如果Alert框命中上方的策略，则此项配置将被跳过。
        self.flag_alert_auto_handled = False

    def swipe_screen(self, x1, y1, x2, y2, count=1):
        '''自由滑动屏幕
        :param x1， y1: 滑动点起点位置（屏幕百分比 0-1）
        :param w2, h2: 滑动点终点位置（屏幕百分比 0-1）
        '''
        for _ in range(count):
            self.device.drag(x1, y1, x2, y2)

    def swipe_screen_oriented(self, x, y, direction, count=1):
        '''沿固定方向滑动屏幕
        :param x， y: 滑动点起点位置（屏幕百分比 0-1）
        :param x2, y2: 滑动点终点位置（屏幕百分比 0-1）
        :param count:     次数
        '''
        x2, y2 = x, y
        if direction == 'right':
            x2 = x + 1
        elif direction == 'left':
            x2 = x - 1
        elif direction == 'top':
            y2 = y + 1
        elif direction == 'bottom':
            y2 = y - 1
        else:
            raise RuntimeError('不支持的方向：%s' % direction)
        for _ in range(count):
            self.device.drag(x, y, x2, y2)
            time.sleep(0.5)

    def swipe_pagedown(self, count=1):
        '''滑动屏幕至下一页
        :param count:     次数
        :type  count:     int
        '''
        for _ in range(count):
            self.device.drag(0.5, 1, 0.5, 0)
            time.sleep(0.5)

    def write_text(self, text):
        self.keyboard.send_keys(text)

    def swipe_pageup(self, count=1):
        '''滑动屏幕至下一页
        :param count:     次数
        :type  count:     int
        '''
        for _ in range(count):
            self._driver.drag(0.5, 0, 0.5, 1)
            time.sleep(0.5)



# -*- coding: utf-8 -*-

from qt4w.webcontrols import *
from qt4i.web import IOSWebView
from qt4i.icontrols import *

'''
Ke WebPage类
'''


class OEDWebPage(WebPage):
    def __init__(self, webview_or_webpage):
        super(OEDWebPage, self).__init__(IOSWebView(webview_or_webpage, QPath('/classname="WebView" & maxdepth=12')))



# -*- coding: UTF-8 -*-

'''Ke App类
'''

import time

from qt4a.androidapp import AndroidApp


class OEDApp(AndroidApp):
    '''
    '''
    package_name = 'com.tencent.mobileqq'
    start_activity = 'com.tencent.mobileqq.activity.SplashActivity'

    def __init__(self, device, kill_process=True, start_activity=True, clear_data=True, choose_user_guide=False):
        super(OEDApp, self).__init__(self.__class__.package_name, device)
        self.re_install = True
        self.choose_user_guide = choose_user_guide

        if kill_process:
            self._device.kill_process(self.__class__.package_name)

        if clear_data:
            self._device.clear_data(self.__class__.package_name)
        else:
            self.re_install = False

        if start_activity:
            self.start()

    def start(self):
        '''启动应用
        '''
        self._device.start_activity(self.__class__.package_name + '/' + self.__class__.start_activity)

        time.sleep(7)



# -*- coding: utf-8 -*-
'''示例测试用例
'''
# 2018/05/16 QTAF自动生成

from testbase.conf import settings

from adapter.device import Device

TESTBASE_CLS = None
if settings.PLATFORM == 'Android':
    from qt4a.androidtestbase import AndroidTestBase

    TESTBASE_CLS = AndroidTestBase

elif settings.PLATFORM == 'iOS':
    from qt4i.itestcase import iTestCase

    TESTBASE_CLS = iTestCase

elif settings.PLATFORM == 'h5':
    from qt4a.androidtestbase import AndroidTestBase

    TESTBASE_CLS = AndroidTestBase
else:
    raise NotImplementedError('Not supported platform %s' % settings.PLATFORM)


class OEDXTestCase(TESTBASE_CLS):
    '''KeX测试用例基类
    '''
    cache = True

    def init_test(self, testresult):
        '''测试用例初始化
        '''
        super(OEDXTestCase, self).init_test(testresult)

    def start_step(self, step):
        '''支持精准测试
        '''
        super(OEDXTestCase, self).start_step(step)

    def acquire_device(self, *args, **kwargs):
        '''申请设备
        '''
        if settings.PLATFORM == 'Android':
            device = super(OEDXTestCase, self).acquire_device(*args, **kwargs)
            device.adb.start_logcat()
            return Device(device)
        elif settings.PLATFORM == 'iOS':
            from qt4i.device import Device as IosDevice
            idev = IosDevice(*args, **kwargs)
            return Device(idev)
        elif settings.PLATFORM == 'h5':
            device = super(OEDXTestCase, self).acquire_device(*args, **kwargs)
            device.adb.start_logcat()
            return Device(device)

    def post_test(self):
        super(OEDXTestCase, self).post_test()



# -*- coding: UTF-8 -*-

from oed_native_lib.OEDWebPage import OEDWebPage


class OEDXWebPage(OEDWebPage):
    '''
    跨平台Ke WebPage类
    '''

    def __init__(self, webview_or_webpage):
        super(OEDXWebPage, self).__init__(webview_or_webpage)



# -*- coding: UTF-8 -*-

'''Ke Native层库
'''

import sys

from testbase.conf import settings

if settings.PLATFORM == 'Android':
    del sys.modules['oed_native_lib']
    import sys
    import oed_android_lib

    sys.modules['oed_native_lib'] = oed_android_lib
elif settings.PLATFORM == 'iOS':
    del sys.modules['oed_native_lib']
    import sys
    import oed_ios_lib

    sys.modules['oed_native_lib'] = oed_ios_lib

elif settings.PLATFORM == 'h5':
    del sys.modules['oed_native_lib']
    import sys
    import oed_h5_lib

    sys.modules['oed_native_lib'] = oed_h5_lib
else:
    raise NotImplementedError('Not supported platform %s' % settings.PLATFORM)



# -*- coding: utf-8 -*-

import math
import time
import urllib2

from metislib.controls import MtControl
from metislib.controls import MtList
from testbase import logger
from testbase.conf import settings

from cache import metis_cache

if settings.PLATFORM == 'iOS':
    from qt4i.icontrols import Element


    class Element(Element):
        def __init__(self, root, locator, **ext):
            super(Element, self).__init__(root, locator, **ext)

        def click(self, offset_x=None, offset_y=None):
            super(Element, self).click(offset_x, offset_y)

        @property
        def text(self):
            return self.value

        @text.setter
        def text(self, text):
            '''设置value(输入，支持中文)
            '''
            self.value = text

        def scroll_to_bottom(self, time=1):
            while time > 0:
                self._app.device.drag(0.5, 0.8, 0.5, 0.5)
                time = time - 1

        def scroll_to_top(self, time=1):
            while time > 0:
                self._app.device.drag(0.5, 0.2, 0.5, 0.5)
                time = time - 1


class MtList(MtList):
    def __init__(self, root, id, region="main", locator=None):
        super(MtList, self).__init__(root, id, region, locator)


class MtControl(MtControl):
    def __init__(self, root, id, locator=None, name=None, instance=None):
        super(MtControl, self).__init__(root, id, locator, name, instance)

    def wait_for_exist(self, timeout=10, interval=0.5):
        '''等待控件出现
         '''
        time0 = time.time()
        while time.time() - time0 < timeout:
            if self.exist():
                return True
            time.sleep(interval)
        return False

    def exist(self):
        time0 = time.time()
        try:
            flag = super(MtControl, self).exist()
        except urllib2.HTTPError:
            logger.info("通过metis判断控件 %s 是否存在, 出现HttpError错误" % self.text)
            return False
        cost = time.time() - time0
        logger.info("通过metis判断控件 %s 是否存在耗时 %.2f 秒" % (self.text, cost))
        return flag

    # def exist(self):
    #     if '*' in self._name:
    #         import re
    #         pattern = re.compile(self._name)
    #         img_file = self._view.screenshot()
    #         driver = self._driver_cls(img_file, self.os_type)
    #         ui_tree = driver.get_uitree()
    #         for item in ui_tree:
    #             if item['Id']:
    #                 if pattern.match(item['Id'].encode('utf-8')):
    #                     return len(driver.find_controls(item['Id'].encode('utf-8'))) > 0
    #         return False
    #     else:
    #         time0 = time.time()
    #         flag = super(MtControl, self).exist()
    #         cost = time.time() - time0
    #         logger.info("通过metis判断控件 %s 是否存在耗时 %.2f 秒" % (self.text, cost))
    #         return flag
    #
    # def _find_control(self, timeout=10):
    #     '''查找控件
    #     '''
    #     正则表达式
    #     if '*' in self._name:
    #         import re
    #         pattern = re.compile(self._name)
    #         time0 = time.time()
    #         while time.time() - time0 < timeout:
    #             try:
    #                 img = self._view.screenshot()
    #                 driver = self._driver_cls(img, self.os_type)
    #                 ui_tree = driver.get_uitree()
    #                 for item in ui_tree:
    #                     if item['Id']:
    #                         if pattern.match(item['Id'].encode('utf-8')):
    #                             return driver.find_control(item['Id'].encode('utf-8'), self._instance)
    #             except ControlNotFoundError as e:
    #                 raise ControlNotFoundError(str(e))
    #             except ControlAmbiguousError:
    #                 self._print_uitree(driver)
    #                 raise
    #             self._print_uitree(driver)
    #     else:
    #         return super(MtControl, self)._find_control(timeout=timeout)

    def _print_uitree(self, driver):
        '''打印窗口的UI树
        '''
        if driver:
            ui_tree = driver.get_uitree()
            lines = []
            for item in ui_tree:
                if item['Id']:
                    _spaces = ''
                    _indent = '|---'
                    _line = _spaces + '{  ' + ',   '.join([
                        'Name: "%s"' % item['Id'],
                        'Type: "%s"' % item['Type'],
                        'Rect: %s' % item['Rect']]) + '  }'
                    lines.append(_line)

    def show_uitree(self):
        '''打印窗口的UI树
        '''
        img = self._view.screenshot()
        driver = self._driver_cls(img, self.os_type)
        if driver:
            ui_tree = driver.get_uitree()
            for item in ui_tree:
                if item['Id']:
                    print('Id: "%s" | Rect: "%s"' % (item['Id'], item['Rect']))

    def _get_uitree(self):
        try:
            uitree = {}
            img = self._view.screenshot()
            driver = self._driver_cls(img, self.os_type)
            if driver:
                ui_tree = driver.get_uitree()
                for item in ui_tree:
                    if item['Id']:
                        uitree[item['Id'].encode('utf-8')] = item['Rect']
            return uitree
        except urllib2.HTTPError:
            logger.info("get_uitree() 出现HttpError错误")
            return False

    def get_uitree(self, timeout=20, interval=0.5):
        '''获取窗口的UI树
        '''
        time0 = time.time()
        while time.time() - time0 < timeout:
            uitree = self._get_uitree()
            if isinstance(uitree, dict):
                return uitree
            time.sleep(interval)
        return {}

    def get_uitree_lastname(self):
        lastname = ""
        img = self._view.screenshot()
        driver = self._driver_cls(img, self.os_type)
        if driver:
            ui_tree = driver.get_uitree()
            for item in ui_tree:
                if item['Id']:
                    lastname = item['Id'].encode('utf-8')
        return lastname

    @metis_cache
    def _get_click_location(self, offset_x=None, offset_y=None):
        '''获取控件的点击的位置的坐标
        '''
        # point_x, point_y = super(MtControl, self)._get_click_location(offset_x=offset_x, offset_y=offset_y)
        # return point_x, point_y
        return self.wrap_get_click_location(offset_x, offset_y)

    def wrap_get_click_location(self, offset_x=None, offset_y=None, timeout=20, interval=2):
        time0 = time.time()
        while time.time() - time0 < timeout:
            point = self._wrap_get_click_location(offset_x, offset_y)
            if isinstance(point, tuple):
                return point[0], point[1]
            time.sleep(interval)
        raise RuntimeError("HttpError")

    def _wrap_get_click_location(self, offset_x=None, offset_y=None):
        try:
            point_x, point_y = super(MtControl, self)._get_click_location(offset_x=offset_x, offset_y=offset_y)
            return point_x, point_y
        except urllib2.HTTPError:
            logger.info("_get_click_location() 出现HttpError错误")
            return False

    def rect_left(self, rect):
        return rect["Left"]

    def rect_right(self, rect):
        return rect["Left"] + rect["Width"]

    def rect_bottom(self, rect):
        return rect["Top"] + rect["Height"]

    def rect_top(self, rect):
        return rect["Top"]

    def _get_relative_name(self, direction):
        try:
            x, y, w, h = self.rect
        except urllib2.HTTPError:
            return False

        left = x
        right = x + w
        top = y
        bottom = y + h
        uitree = self.get_uitree()
        min_value = 10000
        target_name = ""
        if direction == 'right':
            for name, rect in uitree.items():
                # 在右面
                if self.rect_left(rect) > right:
                    if (top < self.rect_top(rect) < bottom) or (top < self.rect_bottom(rect) < bottom):
                        value = self.rect_left(rect) - right
                        if value < min_value:
                            min_value = value
                            target_name = name
            return target_name
        elif direction == 'left':
            for name, rect in uitree.items():
                # 在左面
                if self.rect_right(rect) < left:
                    if (top < self.rect_top(rect) < bottom) or (top < self.rect_bottom(rect) < bottom):
                        value = left - self.rect_right(rect)
                        if value < min_value:
                            min_value = value
                            target_name = name
            return target_name
        elif direction == 'top':
            for name, rect in uitree.items():
                # 在上面
                if self.rect_bottom(rect) < top:
                    if (left < self.rect_left(rect) < right) or (left < self.rect_right(rect) < right):
                        value = top - self.rect_bottom(rect)
                        if value < min_value:
                            min_value = value
                            target_name = name
            return target_name
        elif direction == 'bottom':
            for name, rect in uitree.items():
                # 在下面
                if self.rect_top(rect) > bottom:
                    if (left < self.rect_left(rect) < right) or (left < self.rect_right(rect) < right):
                        value = self.rect_top(rect) - bottom
                        if value < min_value:
                            min_value = value
                            target_name = name
            return target_name
        else:
            raise RuntimeError('不支持的方向：%s' % direction)

    def get_relative_name(self, direction, timeout=20, interval=0.5):
        time0 = time.time()
        while time.time() - time0 < timeout:
            name = self._get_relative_name(direction)
            if isinstance(name, str):
                return name
            time.sleep(interval)
        return ""

    def show_elements_around(self):
        for direction in ['left', 'right', 'top', 'bottom']:
            print("%s: %s" % (direction, self.get_relative_name(direction)))

    def get_control_location(self):
        loc_x, loc_y = super(MtControl, self)._get_click_location()
        return loc_x, loc_y

    def get_control_pose(self):
        loc_x, loc_y = self.get_control_location()
        if settings.PLATFORM == "Android":
            screen_width, screen_height = self._view._device.screen_size
        pose_x, pose_y = loc_x * screen_width, loc_y * screen_height
        return pose_x, pose_y

    def click_relative_pose(self, offset_x, offset_y):
        x, y = self.get_control_location()
        width_ratio, height_ratio = x + offset_x, y + offset_y
        self.wait_for_exist(timeout=5, interval=0.5)
        if settings.PLATFORM == "Android":
            screen_width, screen_height = self._view._device.screen_size
            self._view._device.run_shell_cmd('input tap %d %d' % (
                screen_width * width_ratio, screen_height * height_ratio))
        if settings.PLATFORM == "iOS":
            self._view._device.click(x=width_ratio, y=height_ratio)

    def _get_screen_size(self):
        if settings.PLATFORM == "Android":
            screen_width, screen_height = self._view._device.screen_size
        # TODO ios screen size
        return screen_width, screen_height

    def click_relative_direction(self, direction, offset=0.1):
        offset_x, offset_y = 0, 0
        if direction == "left":
            offset_x -= offset
        elif direction == "right":
            offset_x += offset
        elif direction == "top":
            offset_y -= offset
        elif direction == "bottom":
            offset_y += offset
        else:
            raise Exception("Unknown direction")
        self.click_relative_pose(offset_x, offset_y)

    def get_name_near_location(self, x, y):
        uitree = self.get_uitree()
        width, height = self._get_screen_size()
        px0, py0 = x * width, y * height
        min_distance = float("inf")
        control_name = ''
        for name, rect in uitree.items():
            if rect['Left'] < px0 < rect['Left'] + rect['Width'] and \
                    rect['Top'] < py0 < rect['Top'] + rect['Height']:
                logger.info("Find within control name: %s" % name)
                return name
            else:
                px1 = rect['Left'] + 0.5 * rect['Width']
                py1 = rect['Top'] + 0.5 * rect['Height']
                d = math.sqrt(pow(px1 - px0, 2) + pow(py1 - py0, 2))
                if d < min_distance:
                    min_distance = d
                    control_name = name
        logger.info("Find nearest control name: %s" % control_name)
        return control_name

    def get_name_near_relative_pose(self, offset_x, offset_y):
        x0, y0 = self.get_control_location()
        x, y = x0 + offset_x, y0 + offset_y
        return self.get_name_near_location(x, y)

    def get_control_near_location(self, x, y):
        uitree = self.get_uitree()
        width, height = self._get_screen_size()
        px0, py0 = x * width, y * height
        min_distance = float("inf")
        control = dict()
        for name, rect in uitree.items():
            if rect['Left'] < px0 < rect['Left'] + rect['Width'] and \
                    rect['Top'] < py0 < rect['Top'] + rect['Height']:
                logger.info("Find within control: %s" % name)
                control = {'name': name, 'location': {'x': x, 'y': y}}
                return control
            else:
                px1 = rect['Left'] + 0.5 * rect['Width']
                py1 = rect['Top'] + 0.5 * rect['Height']
                d = math.sqrt(pow(px1 - px0, 2) + pow(py1 - py0, 2))
                if d < min_distance:
                    min_distance = d
                    x, y = px1 / width, py1 / height
                    control = {'name': name, 'location': {'x': x, 'y': y}}
        logger.info("Find nearest control: %s" % control['name'])
        return control

    def get_control_near_relative_pose(self, offset_x, offset_y):
        x0, y0 = self.get_control_location()
        x, y = x0 + offset_x, y0 + offset_y
        return self.get_control_near_location(x, y)



# -*- coding: UTF-8 -*-

import time
from oed_native_lib.OEDWindow import OEDWindow
from adapter.Element import MtControl
from testbase.conf import settings
from testbase import logger
from tuia.exceptions import ControlAmbiguousError


class OEDXWindow(OEDWindow):
    '''
    跨平台 Window类
    '''

    def __init__(self, OEDApp, **kwds):
        OEDWindow.__init__(self, OEDApp, **kwds)

    def updateLocator(self, locators):
        if settings.PLATFORM == "Android" or settings.PLATFORM == "h5":
            super(OEDXWindow, self).update_locator(locators)
        else:
            super(OEDXWindow, self).updateLocator(locators)

    def wait_for_exist(self, timeout=10, interval=0.5):
        if settings.PLATFORM == "h5":
            self.Activity = "com.tencent.mobileqq.activity.QQBrowserActivity"
        try:
            return super(OEDXWindow, self).wait_for_exist(timeout, interval)
        except Exception:
            return False

    def click_screen(self, width_ratio=0.5, height_ratio=0.5):
        self.wait_for_exist(timeout=5, interval=0.5)
        if settings.PLATFORM == "Android":
            screen_width, screen_height = self.device.screen_size  # 获取屏幕宽度、高度
            self.device.run_shell_cmd('input tap %d %d' % (
                screen_width * width_ratio, screen_height * height_ratio))
        if settings.PLATFORM == "iOS":
            self._device.click(x=width_ratio, y=height_ratio)

    def touch_skip(self, timeout=15, interval=0.5):
        """
        点击跳过
        """
        time0 = time.time()
        self.add_control('跳过')
        while time.time() - time0 < timeout:
            if self.Controls['跳过'].exist():
                self.Controls['跳过'].click()
                return True
            time.sleep(interval)

    def click_confirm(self, timeout=10, interval=0.5):
        """
        点击确定
        """
        time0 = time.time()
        self.add_control("确认")
        while time.time() - time0 < timeout:
            if self.Controls['确认'].exist():
                self.Controls['确认'].click()
                return True
            time.sleep(interval)

    def click_cancel(self, timeout=10, interval=0.5):
        """
        点击取消
        """
        time0 = time.time()
        self.add_control("取消")
        while time.time() - time0 < timeout:
            if self.Controls['取消'].exist():
                self.Controls['取消'].click()
                return True
            time.sleep(interval)

    def click_defined_element(self, name, search_page=5):
        """
        在页面查找Métis控件并点击，当前页面找不到就翻到下一页
        需在相应xml中添加对应name(TODO自动更新adapter)
        """
        self.wait_for_exist(timeout=5, interval=2)
        self.Controls[name].wait_for_exist(timeout=5, interval=0.5)
        for _ in range(search_page):
            if self.Controls[name].exist():
                self.Controls[name].click()
                return
            self._app.swipe_pagedown()
        self.Controls[name].show_uitree()
        raise Exception("未找到元素 %s" % name)

    def verify_element_existence(self, name, search_page=1):
        """
        在页面查找Métis控件，默认为当前页，可翻页查找
        """
        self.wait_for_exist(timeout=5, interval=2)
        self.Controls[name].wait_for_exist(timeout=5, interval=0.5)
        for _ in range(search_page):
            if self.Controls[name].exist():
                return True
            self._app.swipe_pagedown()
        return False

    def show_uitree(self):
        '''打印窗口的UI树
        '''
        self.updateLocator({"Mt": {'type': MtControl, 'root': self}})
        self.Controls['Mt'].show_uitree()

    def add_control(self, name):
        '''仅限新增临时control，无需在xml中添加
        '''
        if name not in self._locators.keys():
            self.updateLocator({name: {'type': MtControl, 'root': self}})

    def click_first_of_repeat_control(self, control):
        try:
            self.add_control(control)
            self.click_defined_element(control)
        except ControlAmbiguousError:
            print("重复控件，查找第一个")
            self.updateLocator({control + "-0": {'type': MtControl, 'root': self, 'name': control, 'instance': 0}})
            self.click_defined_element(control + "-0")

    def swipe_pagedown(self):
        logger.info("翻到下一页")
        self._app.swipe_pagedown()

    def swipe_pageup(self):
        logger.info("翻到上一页")
        self._app.swipe_pageup()

    def swipe_to_visible(self, control, page=5):
        self.add_control(control)
        for _ in range(page):
            if self.Controls[control].exist():
                logger.info("Find %s" % control)
                return
            else:
                self.swipe_pagedown()
        logger.error("Not find %s in %s pages" % (control, page))

    def swipe_screen_oriented(self, x, y, direction, count=1):
        '''沿固定方向滑动屏幕
        :param x， y: 滑动点起点位置（屏幕百分比 0-1）
        :param direction: 滑动方向 (left, right, top, bottom)
        :param count:     次数
        '''
        logger.info("向%s方向滑动" % direction)
        self._app.swipe_screen_oriented(x, y, direction, count=1)